// programme de test

#include <iostream>
#include <fstream>
#include "Graphe.h"
#include "Algo.h"
#include "Heuristique.h"

using namespace std;

int main(int argc, char** argv) { 

	Graphe g;
	
	// test pour savoir si le premier argument est un fichier
	ifstream file(argv[1]);
	if (file.is_open()) { 
		// on lit dans un fichier
		g=Graphe(argv[1]);
	}
	else { 
		// aleatoire (eventuellement nombre d'aretes fixees)
		if (argv[1][0]=='a') {
			if (argc<5) g=Graphe(atoi(argv[2])); 
			else g=Graphe(atoi(argv[2]),atoi(argv[3]));	
		}
		// fasciagraphe "pseudo-complet"
		else if (argv[1][0]=='f') g=Graphe(atoi(argv[2]),atoi(argv[3]),false);
		// rotagraphe "pseudo-complet" ou roue
		else if (argv[1][0]=='r') {
			if (argc<5) g=Graphe(atoi(argv[2]),'r');
			else g=Graphe(atoi(argv[2]),atoi(argv[3]),true);
		}
		// cycle
		else if (argv[1][0]=='c') g=Graphe(atoi(argv[2]),'c');
		// etoile
		else if (argv[1][0]=='e') g=Graphe(atoi(argv[2]),'e');
		// graphe dont tous les sommets sont de meme degre
		else if (argv[1][0]=='m') {
			g=Graphe();
			g.build_sd(atoi(argv[2]),atoi(argv[3]));
		}
		// graphe complet dont on a deplace le cycle exterieur
		else if (argv[1][0]=='h') {
			g=Graphe();
			g.build_cc(atoi(argv[2]));
		}
		// toile d'araignee aleatoire; niveau de complexite allant de 0 a 3: fusion/generation
		else if (argv[1][0]=='t') {
			g=Graphe();
			g.build_web(atoi(argv[2]),atoi(argv[3]),atoi(argv[4])); // argv[2]: nombre d'etages
		}
		else {
			cout << endl << "arguments non valides" << endl << endl;
			cout << "<d> etant le seuil de dislocation souhaite, voici les utilisations possibles:" << endl << endl;
			cout << "<nom_de_fichier> <d>: recupere un graphe dans un fichier" << endl << endl;
			cout << "'a' <n> <d>: construit un graphe aleatoire a n sommets" << endl << endl; 
			cout << "'a' <n> <m> <d>: construit un graphe aleatoire a n sommets et m aretes" << endl << endl; 
			cout << "'f' <p> <q> <d>: construit le fasciagraghe a p sous-graphes complets de taille q, totalement connectes" << endl << endl;
			cout << "'r' <p> <q> <d>: construit le rotagraghe a p sous-graphes complets de taille q, totalement connectes" << endl << endl;
			cout << "'r' <n> <d>: construit la roue de velo a n sommets" << endl << endl;
			cout << "'c' <n> <d>: construit le cycle a n sommets" << endl << endl;
			cout << "'e' <n> <d>: construit l'etoile a n sommets" << endl << endl;
			cout << "'m' <n> <k> <d>: construit le graphe a n sommets dont tous les sommets ont pour degre k" << endl << endl;
			cout << "'h' <n> <d>: construit le graphe complet a n sommets dont on a ecarte le cycle exterieur" << endl << endl;
			cout << "'t' <n> <p> <q> <d>: construit une toile d'araignee aleatoire a p niveaux, de complexite q, 0<=q<=3, ou le premier niveau a n sommets" << endl << endl;
			return 0;
		}
	}

	// affichage de la matrice d'adjacence (si pas trop grande)
	if (g.n<=61) cout << "graphe: " << endl << g << endl;

/*** versions approximatives ***/
	
	/*** version 1 ***/
	
	cout << "***** premier algo *****" << endl;
	
	// initialisation de l'algo
	Algo alg1=Algo(g); 
	
	// recherche du nombre "minimal" (approche) de sommets a retirer 
	cout << "il faut enlever au plus ";
	if (argc<4) cout << alg1.a1(atoi(argv[2]),0);
	else if (argc<5) cout << alg1.a1(atoi(argv[3]),0);
	else if (argc<6) cout << alg1.a1(atoi(argv[4]),0);
	else cout << alg1.a1(atoi(argv[5]),0); 
	cout << " sommet(s)" << endl << endl; 
	cout << "une solution possible est:" << endl;
	alg1.affiche(); 
	
	/*** version 2 ***/
	
	cout << endl << "***** second algo *****" << endl;
	
	// initialisation de l'algo
	Algo alg2=Algo(g); 
	
	// recherche du nombre "minimal" (approche) de sommets a retirer 
	cout << "il faut enlever au plus ";
	if (argc<4) cout << alg2.a2(atoi(argv[2]),0);
	else if (argc<5) cout << alg2.a2(atoi(argv[3]),0);
	else if (argc<6) cout << alg2.a2(atoi(argv[4]),0);
	else cout << alg2.a2(atoi(argv[5]),0); 
	cout << " sommet(s)" << endl << endl; 
	cout << "une solution possible est:" << endl;
	alg2.affiche();
	
	/*** version 3 ***/
	
	cout << endl << "***** troisieme algo *****" << endl;
	
	// initialisation de l'algo
	Algo alg3=Algo(g); 
	
	// recherche du nombre "minimal" (approche) de sommets a retirer 
	cout << "il faut enlever au plus ";
	if (argc<4) cout << alg3.a3(atoi(argv[2]),0);
	else if (argc<5) cout << alg3.a3(atoi(argv[3]),0);
	else if (argc<6) cout << alg3.a3(atoi(argv[4]),0);
	else cout << alg3.a3(atoi(argv[5]),0); 
	cout << " sommet(s)" << endl << endl; 
	cout << "une solution possible est:" << endl;
	alg3.affiche();
	
/*** versions probabilistes ***/
	
	/*** version 1 ***/
	
	cout << endl << "***** proba 1 *****" << endl;
	
	// initialisation de l'algo
	Algo alg1p=Algo(g); 
	
	// recherche du nombre "minimal" (approche) de sommets a retirer 
	cout << "il faut enlever au plus ";
	if (argc<4) cout << alg1p.proba1(atoi(argv[2]));
	else if (argc<5) cout << alg1p.proba1(atoi(argv[3]));
	else if (argc<6) cout << alg1p.proba1(atoi(argv[4]));
	else cout << alg1p.proba1(atoi(argv[5])); 
	cout << " sommet(s)" << endl << endl; 
	cout << "une solution possible est:" << endl;
	alg1p.affiche(); 
	
	/*** version 2 ***/
	
	cout << endl << "***** proba 2 *****" << endl;
	
	// initialisation de l'algo
	Algo alg2p=Algo(g); 
	
	// recherche du nombre "minimal" (approche) de sommets a retirer 
	cout << "il faut enlever au plus ";
	if (argc<4) cout << alg2p.proba2(atoi(argv[2]));
	else if (argc<5) cout << alg2p.proba2(atoi(argv[3]));
	else if (argc<6) cout << alg2p.proba2(atoi(argv[4]));
	else cout << alg2p.proba2(atoi(argv[5])); 
	cout << " sommet(s)" << endl << endl;  
	cout << "une solution possible est:" << endl;
	alg2p.affiche();
	
	/*** version 3 ***/
	
	cout << endl << "***** proba 3 *****" << endl;
	
	// initialisation de l'algo
	Algo alg3p=Algo(g); 
	
	// recherche du nombre "minimal" (approche) de sommets a retirer 
	cout << "il faut enlever au plus ";
	if (argc<4) cout << alg3p.proba3(atoi(argv[2]));
	else if (argc<5) cout << alg3p.proba3(atoi(argv[3]));
	else if (argc<6) cout << alg3p.proba3(atoi(argv[4]));
	else cout << alg3p.proba3(atoi(argv[5])); 
	cout << " sommet(s)" << endl << endl;  
	cout << "une solution possible est:" << endl;
	alg3p.affiche();

/*** recuit simule + recherche tabou [en fait probabiliste aussi] ***/

	cout << endl << "***** recuit simule tabou *****" << endl;
	
	// initialisation de l'algo
	Heuristique h=Heuristique(g); 
	
	// recherche "dichotomique" du nombre "minimal" (approche) de sommets a retirer 
	int nbSommets,inf=0,sup;
	bool b=false;
	if (argc<4) {
		nbSommets=(g.n-atoi(argv[2]))/2;
		sup=g.n-atoi(argv[2]);
		while (inf<sup) {
			if (h.rst(atoi(argv[2]),nbSommets)) {
				sup=nbSommets;
				b=true;
			}
			else inf=nbSommets+1;
			nbSommets=(sup+inf)/2;
		}
	}			
	else if (argc<5) {
		nbSommets=(g.n-atoi(argv[3]))/2;
		sup=g.n-atoi(argv[3]);
		while (inf<sup) {
			if (h.rst(atoi(argv[3]),nbSommets)) {
				sup=nbSommets;
				b=true;
			}
			else inf=nbSommets+1;
			nbSommets=(sup+inf)/2;
		}
	}
	else if (argc<6) {
		nbSommets=(g.n-atoi(argv[4]))/2;
		sup=g.n-atoi(argv[4]);
		while (inf<sup) {
			if (h.rst(atoi(argv[4]),nbSommets)) {
				sup=nbSommets;
				b=true;
			}
			else inf=nbSommets+1;
			nbSommets=(sup+inf)/2;
		}
	}
	else {
        nbSommets=(g.n-atoi(argv[5]))/2;
		sup=g.n-atoi(argv[5]);
		while (inf<sup) {
			if (h.rst(atoi(argv[5]),nbSommets)) {
				sup=nbSommets;
				b=true;
			}
			else inf=nbSommets+1;
			nbSommets=(sup+inf)/2;
		}
	}
	if (b) {
		cout << "avec un peu de chance, il faut enlever exactement " << nbSommets << " sommet(s)" << endl << endl;
		cout << "et une solution possible est:" << endl;
		h.affiche(nbSommets);
	}
	else {
		cout << "il suffit bien sur de retirer " << nbSommets << " sommet(s) quelconque(s)" << endl;
		cout << "d'ailleurs je n'ai pas trouve mieux" << endl;
	}
	
/*** algorithme exact (OK pour tous les graphes si n<=14, depend du graphe sinon) ***/

	// initialisation de l'algo
	Algo alg_exact=Algo(g);
	
	cout << endl << "avec l'algo exact:" << endl;
	cout << "le rapport delta*m/k*n vaut ";
	int res;
	if (argc<4) {
		res=alg_exact.res_exact(atoi(argv[2]));
		cout << (float)(atoi(argv[2])*g.m)/(float)(res*g.n) << endl;
	}
	else if (argc<5) {
		res=alg_exact.res_exact(atoi(argv[3]));
		cout << (float)(atoi(argv[3])*g.m)/(float)(res*g.n) << endl;
	}
	else if (argc<6) {
		res=alg_exact.res_exact(atoi(argv[4]));
		cout << (float)(atoi(argv[4])*g.m)/(float)(res*g.n) << endl;
	}
	else {
		res=alg_exact.res_exact(atoi(argv[5]));
		cout << (float)(atoi(argv[5])*g.m)/(float)(res*g.n) << endl;
	}	
	cout << "il faut retirer " << res << " sommet(s)" << endl;
	cout << "par exemple:" << endl;
	alg_exact.affiche();
	
	return 0; 
}	
