#include "Graphe.h"

// calcul du nombre maximal d'elements dans une partie connexe du graphe 
int Graphe::maxConnexe() { // complexite "pseudo-lineaire" (theorie: O(n^2))
	// indices a traiter
	int* indice=(int*)malloc(n*sizeof(int));
	// mise a "zero" du tableau "classe Connexite" 
	for (int i=n-1; i>=0; i--) classeConnexite[i]=-1;
	
	// parcours de la matrice en lignes
	for (int i=n-1; i>=0; i--) { 
		if (classeConnexite[i]<0) { // sinon, deja traite
			indiceAcces=0;
			mark=i; // marque utilisee pour la classe de connexite
			newClasse=true; // indique si on visite une nouvelle classe
			accedes[indiceAcces++]=i;
			classeConnexite[i]=mark;
			for (int j=n-1; j>=0; j--)
				if (tab[i][j] && classeConnexite[j]<0) {
					indice[indiceAcces-1]=j; // "prochain" indice a tester
					classeConnexite[j]=mark;
					accedes[indiceAcces++]=j;
				}
				else if (tab[i][j]) {
					// on rencontre une classe deja connue
					newClasse=false;
					mark=classeConnexite[j];
				}
			
			// on met a jour les sommets visites avec mark si necessaire
			if (!newClasse) 
				for (int j=indiceAcces-1; j>=0; j--) classeConnexite[accedes[j]]=mark;
			// appels recursifs	
			int mem=indiceAcces-2;
			for (int j=mem; j>=0; j--) aux_connex(indice[mem-j]);	
		}	
	}
	
	// on a eventuellement construit des classes de connexite en plusieurs temps
	// il faut maintenant trouver le max (chaque sommet a ete visite a ce stade)
	for (int i=n-1; i>=0; i--) accedes[i]=0;
	mark=-1;
	for (int i=n-1; i>=0; i--)
		if (++accedes[classeConnexite[i]]>mark) mark=accedes[classeConnexite[i]]; 
	return mark;
}
	
// fonction auxiliaire
void Graphe::aux_connex(int s) { 
	int ind_aux=0;
	// indices a traiter
	int* indice=(int*)malloc(n*sizeof(int));
	for (int j=n-1; j>=0; j--)
		if (tab[s][j] && classeConnexite[j]<0) {
			indice[ind_aux++]=j; // "prochain" indice a tester
			classeConnexite[j]=mark;
			accedes[indiceAcces++]=j;
		}
		else if (tab[s][j]) {
			// on rencontre une classe deja connue
			newClasse=false;
			mark=classeConnexite[j];
		}
       
	// on met a jour les sommets visites avec mark si necessaire
	if (!newClasse)
		for (int j=indiceAcces-1; j>=0; j--) classeConnexite[accedes[j]]=mark;
	// appels recursifs
	for (int j=--ind_aux; j>=0; j--) aux_connex(indice[ind_aux-j]);	
	return;
} 
	
// mise a jour des sous-ensembles connexes du graphe 
void Graphe::mAJConnexe() {
	// indices a traiter
	int* indice=(int*)malloc(n*sizeof(int)); 
	// mise a "zero" du tableau "classeConnexite" + indices de connexite + indice
	for (int i=n-1; i>=0; i--) {
		classeConnexite[i]=-1;
		indices_connex[i]=false;
		indice[i]=-1;
	}
	
	// remise a zero des classes de connexite
	for (int i=n-1; i>=0; i--)
		for (int j=n-1; j>=0; j--)
			cl_connex[i][j]=false;
				
	// parcours de la matrice en lignes
	for (int i=n-1; i>=0; i--) { 
		if (classeConnexite[i]<0) { // sinon, deja traite
			indiceAcces=0;
			old_mark=-1,mark=i; // marque utilisee pour la classe de connexite
			newClasse=true; // indique si on visite une nouvelle classe
			accedes[indiceAcces]=i;
			indiceAcces++;
			classeConnexite[i]=mark;
			cl_connex[mark][i]=true;
			indices_connex[i]=true;
			for (int j=n-1; j>=0; j--)
				if (tab[i][j] && classeConnexite[j]<0) {
					indice[indiceAcces-1]=j; // prochain indice a tester
					classeConnexite[j]=mark;
					cl_connex[mark][j]=true;
					accedes[indiceAcces++]=j;
				}
				else if (tab[i][j]) {
					// on rencontre une classe "deja connue"; attention:
					// il faut verifier que l'on est pas retombes sur la classe en construction
					if (mark!=classeConnexite[j]) {
						newClasse=false;
						old_mark=mark;	
						mark=classeConnexite[j];
					}
				}
			
			// on met a jour les sommets visites avec mark si necessaire
			if (!newClasse) {
				indices_connex[i]=false;
				for (int j=indiceAcces-1; j>=0; j--) {
					classeConnexite[accedes[j]]=mark;
					cl_connex[mark][accedes[j]]=true;
					for (int k=n-1; k>=0; k--) cl_connex[old_mark][k]=false;
				}
			}
			// appels recursifs	
			int mem=indiceAcces-2;
			for (int j=mem; j>=0; j--) aux_majC(indice[mem-j]);		
		}	
	}
	return;
}      
	
// fonction auxiliaire
void Graphe::aux_majC(int s) { 
	int ind_aux=0;
	// indices a traiter
	int* indice=(int*)malloc(n*sizeof(int));
	for (int j=n-1; j>=0; j--)
	if (tab[s][j] && classeConnexite[j]<0) {
		indice[ind_aux++]=j; // prochain indice a tester
		classeConnexite[j]=mark;
		cl_connex[mark][j]=true;
		accedes[indiceAcces++]=j;
	}
	else if (tab[s][j]) {
		// on rencontre une classe "deja connue"; attention:
		// il faut verifier que l'on est pas retombes sur la classe en construction
		if (mark!=classeConnexite[j]) {
			newClasse=false;
			old_mark=mark;
			mark=classeConnexite[j];
		}
	}
	
	// on met a jour les sommets visites avec mark si necessaire
	if (!newClasse)
		for (int j=indiceAcces-1; j>=0; j--) classeConnexite[accedes[j]]=mark;
	// appels recursifs 
	for (int j=--ind_aux; j>=0; j--) aux_majC(indice[ind_aux-j]);	
	return;
} 
       			
// supprime un element de degre maximum 
RgName Graphe::remove_max(int p) {
	std::vector<int> max_degre; // ensemble des sommets de degre max
	int max=-1;
	int nb;
	for (int i=n-1; i>=0; i--) {
		nb=0;
		for (int j=n-1; j>=0; j--)
			if (tab[i][j]) nb++;
		if (nb>max) {
			max=nb;
			max_degre.clear();
			max_degre.push_back(i);
		}
		else if (nb>=max) max_degre.push_back(i);
	}
	// choix "aleatoire"
	int val=max_degre[(rand()+p)%(int)max_degre.size()];
	int res=noms[val];
	
	// seconde etape: suppression du sommet --> permutations lignes/colonnes
	if (val<n-1) {
		// colonnes
		// sauvegarde de tab[*][val]
		for (int i=n-1; i>=0; i--) {
			temp[i]=tab[i][val];
			if (temp[i]) m--;
		}	
        
		for (int i=val; i<n-1; i++) 
			for (int j=n-1; j>=0; j--) tab[j][i]=tab[j][i+1];
        
		// restitution dans la derniere colonne
		for (int i=n-1; i>=0; i--) tab[i][n-1]=temp[i];
        
		// lignes
		// sauvegarde de tab[val][*]
		for (int i=n-1; i>=0; i--) temp[i]=tab[val][i];   
            
		for (int i=val; i<n-1; i++) 
			for (int j=n-1; j>=0; j--) tab[i][j]=tab[i+1][j]; 
            
		// restitution dans la derniere ligne
		for (int i=n-1; i>=0; i--) tab[n-1][i]=temp[i];    
        
		// enfin, mise a jour des identifiants
		for (int i=val; i<n-1; i++) noms[i]=noms[i+1];
		noms[n-1]=res; // important pour restituer le sommet
	} 
	else {
		// actualisation de m
		for (int i=n-1; i>=0; i--)
			if (tab[i][n-1]) m--;
	}	
	n--; 
	return RgName(val,res);		
}			

// supprime simplement un element
void Graphe::remove(int s) { 
	// suppression du sommet --> permutations lignes/colonnes
	if (s<n-1) {
		// colonnes
		// sauvegarde de tab[*][s]
		for (int i=n-1; i>=0; i--) {
			temp[i]=tab[i][s];
			if (temp[i]) m--;
		}
        
		for (int i=s; i<n-1; i++) 
			for (int j=n-1; j>=0; j--) tab[j][i]=tab[j][i+1];
        
		// restitution dans la derniere colonne
		for (int i=n-1; i>=0; i--) tab[i][n-1]=temp[i];
        
		// lignes
		// sauvegarde de tab[s][*]
		for (int i=n-1; i>=0; i--) temp[i]=tab[s][i];   
            
		for (int i=s; i<n-1; i++) 
			for (int j=n-1; j>=0; j--) tab[i][j]=tab[i+1][j]; 
            
		// restitution dans la derniere ligne
		for (int i=n-1; i>=0; i--) tab[n-1][i]=temp[i];    
        
		int sav=noms[s];
		// enfin, mise a jour des identifiants
		for (int i=s; i<n-1; i++) noms[i]=noms[i+1]; 
		noms[n-1]=sav;
	} 
	else {
		// actualisation de m
		for (int i=n-1; i>=0; i--)
			if (tab[i][n-1]) m--;
	}
	n--;
	return;
}
    
// donne le rang en fonction du nom
int Graphe::rank(int nm) { 
	for (int i=n-1; i>=0; i--)
		if (noms[i]==nm) return i;
	return -1;
}
    
// restitution du dernier sommet supprime a la position effective (avant suppression) pos
void Graphe::restitue(int pos) {
	// insertion au bon endroit
	if (pos<n) {
		// lignes
		// sauvegarde de tab[n][*]
		for (int i=n; i>=0; i--) {
			temp[i]=tab[n][i];   
			if (temp[i]) m++;
		}
            
		for (int i=n; i>pos; i--) 
			for (int j=n; j>=0; j--) tab[i][j]=tab[i-1][j]; 
            
		// restitution dans la ligne pos
		for (int i=n; i>=0; i--) tab[pos][i]=temp[i]; 
        
		// colonnes
		// sauvegarde de tab[*][n]
		for (int i=n; i>=0; i--) temp[i]=tab[i][n];
        
		for (int i=n; i>pos; i--) 
			for (int j=n; j>=0; j--) tab[j][i]=tab[j][i-1];
        
		// restitution dans la colonne pos
		for (int i=n; i>=0; i--) tab[i][pos]=temp[i];
        
		int sav=noms[n];
		// enfin, mise a jour des identifiants
		for (int i=n; i>pos; i--) noms[i]=noms[i-1]; 
		noms[pos]=sav;
	}
	else {
		// actualisation de m
		for (int i=n; i>=0; i--)
			if (tab[i][n]) m++;
	}
	n++;
	return;
}
    
// affichage de la matrice d'adjacence
std::ostream& operator<<(std::ostream& o, const Graphe& g) {
	for (int i=0; i<g.n; i++) { 
		if (g.noms[i]<10) o << g.noms[i] << ":  ";
		else o << g.noms[i] << ": ";
		for (int j=0; j<g.n; j++) 
			o << g.tab[i][j] << " ";  
			o << std::endl;
	}
	return o;
}
       
