#ifndef _Graphe
#define _Graphe

/*
* Classe pour reprsenter un graphe en memoire: matrice d'adjacence
*/

#include <iostream>
#include <fstream>
#include <vector>
#include <string>

/* df. des fonctions rand(), srand() et de RAND_MAX + time() */
#include <cstdlib>

// rang et identifiant d'un element dans un graphe
struct RgName {
	// rang + nom
	RgName () {}
	RgName (int rg, int nm) {
		rang=rg;
		nom=nm;
	}
	int rang;
	int nom;
};

// structure pour stocker un element + son suivant dans la toile d'araignee
struct Elt {
	Elt (int cr, int sv) {
		cour=cr;
		suiv=sv;
	}
	int cour;
	int suiv;
};

// type pour un vecteur d'Elt
typedef std::vector<Elt> vect_Elt;

class Graphe {
      
public:
       
	/* constructeur vide */
	Graphe() {}

	/* constructeur "aleatoire" attention: la matrice doit etre symetrique */
	Graphe(int n) { 
		this->n=n;
		m=0;
		tab=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) tab[i]=(bool*)malloc(n*sizeof(bool));
		// initialisation du generateur aleatoire
		srand(time(NULL));
		for (int i=n-1; i>=0; i--) {
			for (int j=i-1; j>=0; j--) { 
				if ((rand())%2<=0) {
					tab[i][j]=false;
					tab[j][i]=false;
				}
				else {
					m++; 
					tab[i][j]=true; 
					tab[j][i]=true;
				}
			}
			/* attention tab[i][i]=0 ! */
			tab[i][i]=false;
		}
		
		/* initialisations */
		classeConnexite=(int*)malloc(n*sizeof(int));
		noms=(int*)malloc(n*sizeof(int));
		for (int i=n-1; i>=0; i--) noms[i]=i;
		accedes=(int*)malloc(n*sizeof(int));  
		cl_connex=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) cl_connex[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				cl_connex[i][j]=false;
		indices_connex=(bool*)malloc(n*sizeof(bool)); 
		temp=(bool*)malloc(n*sizeof(bool));
	}
	
	/* constructeur "aleatoire" avec nb d'aretes fixees */
	Graphe(int n, int m) { 
		this->n=n;
		this->m=m;
		int compt=m;
		tab=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) tab[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				tab[i][j]=false;
		// initialisation du generateur aleatoire
		srand(time(NULL));        
		while (compt>0) {
			for (int i=n-2; i>=0; i--) {
				int aux=(rand())%(n-i)+i;
				while (aux<=i) aux=(rand())%(n-i)+i; // on veut aux>i
				if (compt>0) {
					if (!tab[i][aux]) compt--;
					tab[i][aux]=true;
					tab[aux][i]=true;
				}
			}
		}
		
		/* initialisations */
		classeConnexite=(int*)malloc(n*sizeof(int));
		noms=(int*)malloc(n*sizeof(int));
		for (int i=n-1; i>=0; i--) noms[i]=i;
		accedes=(int*)malloc(n*sizeof(int));  
		cl_connex=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) cl_connex[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				cl_connex[i][j]=false;
		indices_connex=(bool*)malloc(n*sizeof(bool));
		temp=(bool*)malloc(n*sizeof(bool));
	}
	
	/* generation du "rotagraphe complet" avec k sous-graphes complets d'ordre p; booleen a false si on veut le fasciagraphe corr 
	en particulier si k=1, on construit K_p */
	Graphe(int k, int p, bool b) { 
		n=k*p;
		if (b) m=(p*k*(k-1))/2+k*p;
		else m=(p*k*(k-1))/2+(k-1)*p;
		tab=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) tab[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				tab[i][j]=false;
		for (int i=n-1; i>=0; i--) {
			for (int j=n-1; j>=0; j--) {
				if (abs(i-j)<=p && i!=j && i/p==j/p) tab[i][j]=true;
				if (i+p==j) tab[i][j]=true;
				if (i-p==j) tab[i][j]=true;
				if (i>=(k-1)*p && j==(i%p) && b) tab[i][j]=true;
				if (i<p && j==(k-1)*p+(i%p) && b) tab[i][j]=true;
			}
		}
		
		/* initialisations */
		classeConnexite=(int*)malloc(n*sizeof(int));
		noms=(int*)malloc(n*sizeof(int));
		for (int i=n-1; i>=0; i--) noms[i]=i;
		accedes=(int*)malloc(n*sizeof(int));  
		cl_connex=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) cl_connex[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				cl_connex[i][j]=false;
		indices_connex=(bool*)malloc(n*sizeof(bool));
		temp=(bool*)malloc(n*sizeof(bool));
		// initialisation du generateur aleatoire
		srand(time(NULL));
	}
	
	/* construction du cycle d'ordre n, de l'etoile ou de la roue */
	Graphe(int n, char t) { 
		this->n=n;
		if (t=='c') m=n;
		else if (t=='e') m=n-1;
		else m=2*n-1;
		tab=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) tab[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				tab[i][j]=false;
		for (int i=n-1; i>=0; i--) {
			for (int j=n-1; j>=0; j--) { 
				if (t=='c') {
					if (j==i+1 || (j==0 && i==n-1)) {
						tab[i][j]=true;
						tab[j][i]=true;
					}
				}
				if (t=='e' || t=='r') {
					if ((i==0 && j>0) || (j==0 && i>0)) {
						tab[i][j]=true;
						tab[j][i]=true;
					}
				}
				if (t=='r') {
					if (i==j+1 || i==j-1 || (i==0 && j==n-1) || (i==n-1 && j==0)) {
						tab[i][j]=true;
						tab[j][i]=true;
					}
				}
			}
		}
		
		/* initialisations */
		classeConnexite=(int*)malloc(n*sizeof(int));
		noms=(int*)malloc(n*sizeof(int));
		for (int i=n-1; i>=0; i--) noms[i]=i;
		accedes=(int*)malloc(n*sizeof(int));  
		cl_connex=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) cl_connex[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				cl_connex[i][j]=false;
		indices_connex=(bool*)malloc(n*sizeof(bool));
		temp=(bool*)malloc(n*sizeof(bool));
		// initialisation du generateur aleatoire
		srand(time(NULL));
	}
			
	/* constructeur par recopie */
	Graphe(const Graphe& g) { 
		n=g.n;
		m=0;
		tab=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) tab[i]=(bool*)malloc(n*sizeof(bool)); 
		for (int i=n-1; i>=0; i--) 
			for (int j=n-1; j>=0; j--) { 
				tab[i][j]=g.tab[i][j]; 
			if (g.tab[i][j]) m++; 
		}
		m=m/2;  	
	  
		/* autres initialisations */
		classeConnexite=(int*)malloc(n*sizeof(int));
		noms=(int*)malloc(n*sizeof(int)); 
		for (int i=n-1; i>=0; i--) noms[i]=g.noms[i]; 
		accedes=(int*)malloc(n*sizeof(int)); 
		cl_connex=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) cl_connex[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--) 
				cl_connex[i][j]=g.cl_connex[i][j]; 
		indices_connex=(bool*)malloc(n*sizeof(bool)); 
		temp=(bool*)malloc(n*sizeof(bool));
		// initialisation du generateur aleatoire
		srand(time(NULL));
	}

	/* constructeur a partir d'un fichier (format n lignes n colonnes) */
	Graphe (char* name) {
		std::ifstream file(name); 
		std::string line;
		n=0;
		m=0;
		int ligne=0;
		while (getline(file,line,'\n')) { 
			if (n<=0) {
				/* on lit tous les caracteres que l'on place dans un temporaire (n est inconnu) */ 
				std::vector<bool> temp;
				for (int i=0;;i++) {
					if (line[i]=='0') temp.push_back(false);
					else if (line[i]=='1') temp.push_back(true);
					else break;
				}
				n=(int)temp.size();
				/* on replace ce temporaire dans tab, alloue avec une taille n*n */
				tab=(bool**)malloc(n*sizeof(bool*));
				for (int i=n-1; i>=0; i--) tab[i]=(bool*)malloc(n*sizeof(bool));
				for (int i=n-1; i>=0; i--) {
					tab[0][i]=temp[i];
					if (temp[i]) m++;
				}
				ligne++;
			}
			else {
				/* on lit tous les caracteres que l'on place dans tab (on s'arrete a \n ou espace, ou autre chose) */
				for (int i=0; i<n; i++) {
					if (line[i]=='0') tab[ligne][i]=false;
					else if (line[i]=='1') {
						tab[ligne][i]=true;
						m++;
					}
					else break;
				}
				ligne++;	
			}
		}
	  	m=m/2;
	  
		/* initialisations */
		classeConnexite=(int*)malloc(n*sizeof(int));
		noms=(int*)malloc(n*sizeof(int));
		for (int i=n-1; i>=0; i--) noms[i]=i;
		accedes=(int*)malloc(n*sizeof(int));
		cl_connex=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) cl_connex[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				cl_connex[i][j]=false; 
		indices_connex=(bool*)malloc(n*sizeof(bool));
		temp=(bool*)malloc(n*sizeof(bool));
		// initialisation du generateur aleatoire
		srand(time(NULL));
	}

	// construction du graphe dont tous les sommets sont de meme degre
	void build_sd(int n, int k) {
		this->n=n;
		m=(n*k)/2;
		// remplissage de tab
		tab=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) tab[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				tab[i][j]=false;
		if (k&1) {
			// k impair
			for (int i=n-1; i>0; i-=2) {
				tab[i][i-1]=true;
				tab[i-1][i]=true;
			}
			int nb=2,deg=1;
			while (deg<k) {
				for (int i=n-1; i>=0; i--) {
					tab[i][(i+nb)%n]=true;
					tab[(i+nb)%n][i]=true;
				}
				nb++;
				deg+=2;
			}    
		}
		else {
			int nb=1,deg=0;
			while (deg<k) {
				for (int i=n-1; i>=0; i--) {
					tab[i][(i+nb)%n]=true;
					tab[(i+nb)%n][i]=true;
				}
				nb++;
				deg+=2;
			}      
		}   
                  
		/* initialisations */
		classeConnexite=(int*)malloc(n*sizeof(int));
		noms=(int*)malloc(n*sizeof(int));
		for (int i=n-1; i>=0; i--) noms[i]=i;
		accedes=(int*)malloc(n*sizeof(int));  
		cl_connex=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) cl_connex[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				cl_connex[i][j]=false;
		indices_connex=(bool*)malloc(n*sizeof(bool));
		temp=(bool*)malloc(n*sizeof(bool));
		// initialisation du generateur aleatoire
		srand(time(NULL));  
		return;
	}

	// construction du graphe complet dont on a deplace le cycle exterieur
	void build_cc(int nn) {
		n=2*nn;
		m=(nn*(nn+1))/2;
		// remplissage de tab
		tab=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) tab[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				tab[i][j]=false;
		for (int i=n-1; i>=0; i--) 
			if (i==nn) {
				tab[i][i+1]=true;
				tab[i+1][i]=true;
				tab[i][n-1]=true;
				tab[n-1][i]=true;
				tab[i][0]=true; 
				tab[0][i]=true; 
			}
			else if (i==n-1) {
				tab[i][nn]=true;
				tab[nn][i]=true;
				tab[i][i-1]=true;
				tab[i-1][i]=true;
				tab[i][i-nn]=true; 
				tab[i-nn][i]=true; 
			}  
			else if (i>nn) {
				tab[i][i+1]=true;
				tab[i+1][i]=true;
				tab[i][i-1]=true;
				tab[i-1][i]=true;
				tab[i][i-nn]=true; 
				tab[i-nn][i]=true;   
			}
			else { // i<=nn-1: presque tout connecte
				for (int j=nn-1; j>=0; j--)
					if (j!=i && abs(j-i)!=1 && abs(j-i)!=nn-1) {
						tab[i][j]=true;
						tab[j][i]=true;
					} 
				tab[i][nn+i]=true; 
				tab[nn+i][i]=true;        
			}    
         
		/* initialisations */
		classeConnexite=(int*)malloc(n*sizeof(int));
		noms=(int*)malloc(n*sizeof(int));
		for (int i=n-1; i>=0; i--) noms[i]=i;
		accedes=(int*)malloc(n*sizeof(int));  
		cl_connex=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) cl_connex[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				cl_connex[i][j]=false;
		indices_connex=(bool*)malloc(n*sizeof(bool));
		temp=(bool*)malloc(n*sizeof(bool));
		// initialisation du generateur aleatoire
		srand(time(NULL)); 
		return;
	}   
        
	// construction de la toile d'araignee aleatoire; niveau de complexite allant de 0 a 3: fusion/generation
	// on se restreint a des noeuds de degre<=3 (sauf le centre)
	// generalisation facile; utile?!
	void build_web(int nn, int p, int q) { // p: nombre d'etages
        	// idee:
		// on demarre avec n sommets, on construit les relations au cycle suivant..etc
		int* tailles=(int*)malloc(p*sizeof(int)); // tableau des tailles des niveaux successifs
		tailles[0]=nn;
		vect_Elt* contenus=(vect_Elt*)malloc((p-1)*sizeof(vect_Elt)); // tableau des relations temporaires
		// initialisation du generateur aleatoire
		srand(time(NULL)); 
		for (int i=0; i<p-1; i++) {
			// generations des liaisons de i a i+1
			int elt_cour=0; // element courant dans le niveau suivant
			for (int j=0; j<tailles[i]; j++) {
				if (q<=0) {
					// liaisons simple, toujours la meme taille 
					contenus[i].push_back(Elt(j,j));
					elt_cour++;
				}
				else if (q==1) {
					// on s'autorise la fusion, mais pas la generation: toiles tres simples aussi
					// tirage aleatoire pour connaitre le type de liaison
					if (rand()&1 && j<tailles[i]-1) {
						// j fusionne avec j+1
						contenus[i].push_back(Elt(j,elt_cour));
						contenus[i].push_back(Elt(j+1,elt_cour++));
						j++;
					}
					else 
						// liaison normale
						contenus[i].push_back(Elt(j,elt_cour++));
				}
				else if (q==2) {
					// on s'autorise la generation, mais pas la fusion: toiles deviennent plus complexes
					// tirage aleatoire pour connaitre le type de liaison
					if (rand()&1) {
						// j engendre deux sommets
						contenus[i].push_back(Elt(j,elt_cour++));
						contenus[i].push_back(Elt(j,elt_cour++));
					}
					else 
						// liaison normale
						contenus[i].push_back(Elt(j,elt_cour++));
				}
				else {
					// tout est permis
					// tirage aleatoire pour connaitre le type de liaison
					int typ=rand()%3;
					if (typ==1 && j<tailles[i]-1) {
						// j fusionne avec j+1
						contenus[i].push_back(Elt(j,elt_cour));
						contenus[i].push_back(Elt(j+1,elt_cour++));
						j++;
					}
					else if (typ==2) {
						// j engendre deux sommets
						contenus[i].push_back(Elt(j,elt_cour++));
						contenus[i].push_back(Elt(j,elt_cour++));
					}
					else 
						// liaison normale
						contenus[i].push_back(Elt(j,elt_cour++));
				}
			}
			tailles[i+1]=elt_cour;
		}
		std::cerr << "coucou" << std::endl;
		// remplissage de tab en fonction des donnees obtenus
		n=1;
		for (int i=0; i<p; i++) n+=tailles[i];
		tab=(bool**)malloc(n*sizeof(bool*));
		for (int i=0; i<n; i++) tab[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=0; i<n; i++)
			for (int j=0; j<n; j++)
				tab[i][j]=false;  
		// liaisons centre-->premier niveau (num 0)			
		for (int i=0; i<nn; i++) {
			tab[0][i+1]=true;
			tab[i+1][0]=true;
		}  
		int ajout=1; // truc a ajouter aux tailles
		for (int i=0; i<p-1; i++) { 
			// parcours de contenus + remplissage
			for (int j=0; j<(int)contenus[i].size(); j++) { 
				tab[ajout+contenus[i][j].cour][ajout+tailles[i]+contenus[i][j].suiv]=true;
				tab[ajout+tailles[i]+contenus[i][j].suiv][ajout+contenus[i][j].cour]=true; 
			}
			ajout+=tailles[i];
		} 
		// liaisons cycliques sur chaque niveau
		ajout=1;
		for (int i=0; i<p; i++) {
			for (int j=0; j<tailles[i]; j++) { 
				if (j==0 && tailles[i]>1) {
					tab[ajout][ajout+1]=true;
					tab[ajout+1][ajout]=true;
					tab[ajout][ajout+tailles[i]-1]=true;
					tab[ajout+tailles[i]-1][ajout]=true;
				}
				else if (j==tailles[i]-1 && tailles[i]>1) { 
					tab[ajout+tailles[i]-1][ajout+tailles[i]-2]=true;
					tab[ajout+tailles[i]-2][ajout+tailles[i]-1]=true;
					tab[ajout+tailles[i]-1][ajout]=true;
					tab[ajout][ajout+tailles[i]-1]=true;
				}
				else if (tailles[i]>1) {
					tab[ajout+j][ajout+j+1]=true;
					tab[ajout+j+1][ajout+j]=true;
					tab[ajout+j][ajout+j-1]=true;
					tab[ajout+j-1][ajout+j]=true;
				}
			}
			ajout+=tailles[i];
		}
		
		/* initialisations */
		m=0;
		for (int i=0; i<n; i++)
			for (int j=0; j<n; j++) 
				if (tab[i][j]) m++;
		m=m/2;
		classeConnexite=(int*)malloc(n*sizeof(int));
		noms=(int*)malloc(n*sizeof(int));
		for (int i=n-1; i>=0; i--) noms[i]=i;
		accedes=(int*)malloc(n*sizeof(int));  
		cl_connex=(bool**)malloc(n*sizeof(bool*));
		for (int i=n-1; i>=0; i--) cl_connex[i]=(bool*)malloc(n*sizeof(bool));
		for (int i=n-1; i>=0; i--)
			for (int j=n-1; j>=0; j--)
				cl_connex[i][j]=false;
		indices_connex=(bool*)malloc(n*sizeof(bool));
		temp=(bool*)malloc(n*sizeof(bool));
		return;
        }

	/* fonctions de mise a jour des champs (nombre de cycles, tailles des chaines/cycles, nombre maximal d'elements 
	dans une partie connexe, ..etc */

	/* calcul du nombre maximal d'elements dans une partie connexe du graphe */
	int maxConnexe();
	
	/* auxiliaire */
	void aux_connex(int s);
	
	/* mise a jour des sous-ensembles connexes du graphe */
	void mAJConnexe();	
	
	/* auxiliaire */
	void aux_majC(int s);
	
	/* suprime un element de degre maximum, renvoie son identifiant+rang */
	RgName remove_max(int p);

	/* supprime un element */
	void remove(int s);

	// donne le rang en fonction du nom
	int rank(int nm);

	// restitution d'un sommet de rang donne
	void restitue(int pos);

	/* destructeur */
	~Graphe() { /*
		for (int i=n-1; i>=0; i--) {
			free(tab[i]);
			free(cl_connex[i]);
		}
		free(tab);
		free(cl_connex);
		free(classeConnexite);
		free(sommes);
		free(noms);
		free(accedes);
		free(indices_connex); --> segFault */
	}

	/* taille en nombre de lignes de la matrice (courante et initiale) */
	int n;
       
	/* et en nombre d'aretes */
	int m;
       
	/* tableau de booleens representant le graphe */
	bool** tab; 

	/* memoire pour la suppression/restitution */
	bool* temp;

	/* tableaux utile pour l'algorithme de recherche du plus grand nb d'elts dans une partie connexe */
	int* classeConnexite;

	/* identifiants des sommets */
	int* noms;
	
	/* memorise les sommets recemment accedes et non deja traites */
	int* accedes;
	
	/* indice courant pour les sommets accedes */
	int indiceAcces;
	
	/* classes de connexite du graphe */
	bool** cl_connex;
	
	/* marque utilisee pour la classe de connexite */
	int old_mark,mark; 
	
	/* indique si on visite une nouvelle classe */
	bool newClasse;
	
	/* indices correspondants aux classes non nulles */
	bool* indices_connex;  
	   
};

/* envoi d'un graphe vers un flux */
extern std::ostream& operator<<(std::ostream& o, const Graphe &g);

#endif
       
       
