#include <cmath>
#include "Heuristique.h"
#include "random-singleton.h"

// un premier algorithme: recuit simule + rcherche tabou
bool Heuristique::rst(int k, int p) {
	
	/*
	std::cout << "||||| debut d'un r-s-t: |||||" << std::endl;
	std::cout << "objectif de la fonction d'eval: " << "<=" << k+1 << std::endl << std::endl;
	*/
    
	// initialisation du generateur aleatoire
	Random::Randomize(k*p);
	// reinitialisation des bons sommets
	for (int i=g.n-1; i>=0; i--) bons_sommets[i]=0;

	// renvoie true s'il existe effectivement une solution avec p sommets
	// sinon: "boucle" (!) [cas d'arret artificiel sur le temps permet la terminaison]
         
	// cas extreme: p==0
	if (p<=0) return (g.maxConnexe()<=k);
	 
	// initialisation avec les p sommets de degrs maximaux
	// [raffinement: essayer avec les plus grandes classes de connexite]
	sol_cour=(int*)malloc(p*sizeof(int));
	mem_suppr=(int*)malloc(p*sizeof(int));
	for (int i=g.n-1; i>=0; i--) bool_cour[i]=false;
	int compt=p;
	RgName aux; 
	// [eventuelle meilleure strategie: choisir les sommets au hasard total!]
	while (compt>0) { 
		aux=g.remove_max(compt);
		mem_suppr[--compt]=aux.rang;
		sol_cour[compt]=aux.nom;	
		bons_sommets[aux.nom]++; 	
		bool_cour[sol_cour[compt]]=true; 
	}	 	

	g.mAJConnexe();
	int somme=0,old_nbC=0; // old_nbC==nombre de classes de connexite sur le graphe precedent
	int max=-1,nbClasses=0;	
	for (int i=g.n-1; i>=0; i--) {
		compteur[i]=0;
		if (g.indices_connex[i]) old_nbC++;
	}
	compt=-1;
	for (int i=g.n-1; i>=0; i--)
		if (++compteur[g.classeConnexite[i]]>compt) compt=compteur[g.classeConnexite[i]]; 
	
	// restitution du graphe pour la poursuite de l'algo
	for (int i=p-1; i>=0; i--) g.restitue(mem_suppr[p-i-1]);

	// affiche la sol_cour + valeur  l'utilisateur (qui peut choisir d'interrompre le programme)
	// affichage pour utilisateur [si desire]
	/*
	std::cout << "essai en cours:" << std::endl;
	for (int i=0; i<p-1; i++) std::cout << sol_cour[i] << ", ";
	if (p>0) std::cout << sol_cour[p-1] << std::endl;
	std::cout << "et son evaluation: " <<  eval << std::endl << std::endl;
	*/
	
	// cas d'arret deterministe
	if (compt<=k) {
		best=(int*)malloc(p*sizeof(int));
		for (int i=p-1; i>=0; i--) best[i]=sol_cour[i];
		return true;
	}
	 
	// variables pour les iterations successives
	int choix, temps=g.n, compte=1; // initialisation empirique..
	int coeff=3; // de meme: nombre d'iterations==coeff*n
	float bonus,eval,moy,dispersion;

	int n=g.n;
	while (true) { 
         
		// cas d'arret sur le temps
		if (temps<=0) break;
	 
		// on tire au hasard des voisinages obtenus par permutations de "true" et "false"
		// si T est le nb de "true", il y en a (n-T)*T, donc O(p^2 | n), lineaire si p est fixe
		// avec une proba P  dfinir on excute:
		// voisinage.push_back(...);
		voisinage.clear(); 
		for (int i=p-1; i>=0; i--)
			for (int j=n-1; j>=0; j--) 
				if (!bool_cour[j]) {  
					
					// a) verification si la paire a ete rencontree recemment
					if ((tab_sol[0].nom1==sol_cour[i] && tab_sol[0].nom2==g.noms[j])
						|| (tab_sol[0].nom2==sol_cour[i] && tab_sol[0].nom1==g.noms[j])
						|| (tab_sol[1].nom1==sol_cour[i] && tab_sol[1].nom2==g.noms[j])
						|| (tab_sol[1].nom2==sol_cour[i] && tab_sol[1].nom1==g.noms[j])
						|| (tab_sol[2].nom1==sol_cour[i] && tab_sol[2].nom2==g.noms[j])
						|| (tab_sol[2].nom2==sol_cour[i] && tab_sol[2].nom1==g.noms[j]))
					continue;
					// shift de tab_sol   
					tab_sol[0].nom1=tab_sol[1].nom1;
					tab_sol[0].nom2=tab_sol[1].nom2;
					tab_sol[1].nom1=tab_sol[2].nom1;
					tab_sol[1].nom2=tab_sol[2].nom2;
					tab_sol[2].nom1=sol_cour[i];
					tab_sol[2].nom2=g.noms[j];
            
					// b) ajout dans le voisinage [+ deja rencontre] avec proba liee a la valeur de la solution, et au temps ecoule
					// on evalue comme precedemment
					// eval=g.maxConnexe()+somme normalisee des card(connex)+terme de dispersion prp a la moyenne)
	            
					// mise a jour de g
					g.remove(j); 
					for (int l=p-1; l>=0; l--) 
						if (l!=i) {
							mem_suppr[l]=g.rank(sol_cour[l]);
							g.remove(mem_suppr[l]); 
						}
					g.mAJConnexe();
 		
					somme=0;
					max=-1;
					nbClasses=0;
					// somme des cardinaux des parties connexes, calcul du cardinal maximal d'une partie connexe
					for (int u=g.n-1; u>=0; u--)
						if (g.indices_connex[u]) {
							nbClasses++;
							compteur[u]=0;
							for (int v=g.n-1; v>=0; v--)
								if (g.cl_connex[u][v]) {
									compteur[u]++;
									somme++;
								}
							if (compteur[u]>max) max=compteur[u];	
						}
						
					// verification que le nombre de classes de connexite augmente, 
					// puis notation de la solution en fonction du resultat
					if (nbClasses>old_nbC) {
						compt=g.n;
						for (int j=g.n-1; j>=0; j--) 
							if (g.indices_connex[j]) {
								nbClasses=0;
								for (int k=g.n-1; k>=0; k--) 
									if (g.cl_connex[j][k]) nbClasses++;
								if (nbClasses<compt) compt=nbClasses;	
							}
						eval=-(float)compt/(float)(3*g.n); // 3: coeff empirique..
					}
					else eval=0;
						
					// calcul des ecarts a la moyenne
					moy=(float)somme/nbClasses;
					dispersion=0;
					for (int u=g.n-1; u>=0; u--) 
						if (g.indices_connex[u]) dispersion+=(compteur[u]-moy)*(compteur[u]-moy);
					// on en deduit l'ecart type empirique, puis l'evaluation: ~<1 car max>0 pour p<g.n, et ecart_type<=maximum(moy,max-moy)
					if (moy>=max-moy) eval+=max+(float)(somme-max)/somme+sqrt(dispersion)/(3*moy); // coefficient empirique ici aussi.. 
					else eval+=max+(float)g.m/(float)(3*g.n*g.n)-(float)(somme-max)/(float)(3*somme)+sqrt(dispersion)/(3*(0.1+max-moy));
					
					// restitution de g
					for (int l=0; l<p; l++) 
						if (l!=i) g.restitue(mem_suppr[l]); 
					g.restitue(j); 
				
					if (eval<k+1) {
						// on a alors trouve une solution admissible
						sol_cour[i]=j;
						best=(int*)malloc(p*sizeof(int));
						for (int u=p-1; u>=0; u--) best[u]=sol_cour[u];
						return true;
					}
				
					// calcul du bonus/malus en fonction de la frequence d'apparition de i, j dans sol_cour
					somme=0;
					for (int u=g.n-1; u>=0; u--) somme+=bons_sommets[u];
					if (somme<7) bonus=(float)(bons_sommets[j]-bons_sommets[i])/7;
					else bonus=(float)(bons_sommets[j]-bons_sommets[i])/somme;
					
					// 7 est un coefficient a ajuster egalement..
					if (voisinage.empty() || Random::Uniform<double>(0, 1)<=bonus/7+exp(-(eval-k-1)/(1+temps))) // terme exp(-(k+1)): normalisation
						// plus on avance dans le temps, moins on choisit de sommets dans le voisinage --> la temperature baisse
						voisinage.push_back(VEval(i,j,eval)); 
				}
	 
		// on choisit dans le voisinage en fonction de proba/temprature/recherche tabou
		// on met  jour sol_cour + son tableau d'indices (trs peu de modif a faire)
		// le plus violent tant de recalculer les classes de connexit  chaque tour.
		choix=Random::Uniform<int>(0,(int)voisinage.size()-1);
		// [eventuellement une reselection en fonction de l'evaluation]
	 	
		// mise a jour de bool_cour et sol_cour
		bool_cour[sol_cour[voisinage[choix].ind1]]=false; 
		bool_cour[voisinage[choix].ind2]=true; 
		sol_cour[voisinage[choix].ind1]=voisinage[choix].ind2;
		bons_sommets[voisinage[choix].ind2]++;
		
		// affichage pour utilisateur [si desire] 
		/*
		std::cout << "essai en cours:" << std::endl;
		for (int i=0; i<p-1; i++) std::cout << sol_cour[i] << ", ";
		if (p>0) std::cout << sol_cour[p-1] << std::endl;
		std::cout << "et son evaluation: " <<  voisinage[choix].eval << std::endl << std::endl;
		*/
	 
	 	if (compte%coeff==0) temps--;
		compte++; 
	}
	return false;
}
         
// affichage d'une solution 
void Heuristique::affiche(int p) { 
	for (int i=0; i<p-1; i++) std::cout << best[i] << ", ";
	if (p>0) std::cout << best[p-1] << std::endl;
	return;
}

