#ifndef _Heuristique
#define _Heuristique

/*
* Classe implementant quelques heuristiques de k-dislocation
*/

#include <iostream>
#include <vector>
#include "Graphe.h"

struct Permut {
	// structure pour stocker une permutation
	Permut (int n1, int n2) { 
		nom1=n1;
		nom2=n2;
	}
	int nom1;
	int nom2;
};

struct VEval {
	// structure pour stocker un voisinage (permutation) + son evaluation
	VEval (int i1, int i2, float ev) { // ind1=indice dans sol_cour; ind2=indice dans le graphe
		ind1=i1;
		ind2=i2;
		eval=ev;
	}
	int ind1;
	int ind2;
	float eval;
};

class Heuristique {

public:
	
	// constructeur vide
	Heuristique() {}

	// construction a partir d'un graphe, comme pour Algo
	Heuristique(const Graphe& g) {
		this->g=Graphe(g);
		compteur=(int*)malloc((g.n)*sizeof(int)); 
		dejaTraite=(bool*)malloc(g.n*sizeof(bool));
		bool_cour=(bool*)malloc(g.n*sizeof(bool));
		bons_sommets=(int*)malloc(g.n*sizeof(int));
		tab_sol=(Permut*)malloc(3*sizeof(Permut));
		tab_sol[0]=Permut(0,0);
		tab_sol[1]=Permut(0,0);
		tab_sol[2]=Permut(0,0);
	}
	
	// recuit simule + recherche tabou
	bool rst(int k, int p);

	// affichage d'une solution 
	void affiche(int p);
	
	// destructeur
	~Heuristique() { /*
		free(sol_cour);
		free(best);
		free(bool_cour);
		free(dejaTraite);
		free(compteur); --> segFault */
	}

private:

	// graphe
	Graphe g;
	
	// tableau des 3 dernieres permutations essayees (avec leurs noms)
	// [on peut aussi penser a stocker un tableau des permutations "qui marchent bien"]
	Permut* tab_sol;
	
	// memorisation des sommets "qui marchent bien" --> bonus
	// on compte le nombre de fois ou ils apparaissent dans une solution
	int* bons_sommets;
	
	// les indices a "true" dans l'essai courant
	int* sol_cour;
	
	// les booleens correspondants
	bool* bool_cour;
	
	// les indices d'echange pour les elements supprimes
	int* mem_suppr;
	
	// les indices a true dans la meilleure solution trouvee
	int* best;
			
	// voisinage d'un vecteur binaire de taille n + evaluation
	std::vector<VEval> voisinage; 
	
	/* tableau pour stocker les elements deja traites */
	bool* dejaTraite;
       
	/* tableau utile a plusieurs reprises */
	int* compteur;
       
};

#endif
